@@ -12,42 +12,26 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import itertools
-from typing import Any, Iterator, List
+from typing import Any, cast, Iterable, Iterator, List, Sized, Union
 
 import torch
+from torch import Tensor
 from torch.nn.parallel import DistributedDataParallel
 from torch.utils.data import BatchSampler, DistributedSampler, Sampler
 
-import pytorch_lightning as pl
 from pytorch_lightning.overrides.base import _LightningModuleWrapperBase
 from pytorch_lightning.utilities import rank_zero_deprecation
 
 
 class LightningDistributedModule(_LightningModuleWrapperBase):
-    def __init__(self, pl_module: "pl.LightningModule") -> None:
-        """Wraps the user's LightningModule and redirects the forward call to the appropriate method, either
-        ``training_step``, ``validation_step``, ``test_step`` or ``predict``. This class is used in combination
-        with :class:`~torch.nn.parallel.DistributedDataParallel` as shown in the example.
-
-        Example:
-
-            ddp_model = torch.nn.parallel.DistributedDataParallel(
-                module=LightningDistributedModule(lightning_module),
-                device_ids=[local_rank],
-                ...
-            )
-
-        Args:
-            pl_module: the model to wrap
-        """
-        super().__init__(pl_module)
+    ...
 
 
-def _find_tensors(obj):  # pragma: no-cover
-    r"""
-    Recursively find all tensors contained in the specified object.
-    """
-    if isinstance(obj, torch.Tensor):
+def _find_tensors(
+    obj: Union[Tensor, list, tuple, dict, Any]
+) -> Union[List[Tensor], itertools.chain]:  # pragma: no-cover
+    """Recursively find all tensors contained in the specified object."""
+    if isinstance(obj, Tensor):
         return [obj]
     if isinstance(obj, (list, tuple)):
         return itertools.chain(*map(_find_tensors, obj))
@@ -59,27 +43,27 @@ def _find_tensors(obj):  # pragma: no-cover
 # In manual_optimization, we need to call reducer prepare_for_backward.
 # Note: Keep track of Pytorch DDP and update if there is a change
 # https://github.com/pytorch/pytorch/blob/v1.7.1/torch/nn/parallel/distributed.py#L626-L638
-def prepare_for_backward(model: DistributedDataParallel, output: Any):
+def prepare_for_backward(model: DistributedDataParallel, output: Any) -> None:
     # `prepare_for_backward` is `DistributedDataParallel` specific.
     if not isinstance(model, DistributedDataParallel):
         return
     if torch.is_grad_enabled() and model.require_backward_grad_sync:
-        model.require_forward_param_sync = True
+        model.require_forward_param_sync = True  # type: ignore[assignment]
         # We'll return the output object verbatim since it is a freeform
         # object. We need to find any tensors in this object, though,
         # because we need to figure out which parameters were used during
         # this forward pass, to ensure we short circuit reduction for any
         # unused parameters. Only if `find_unused_parameters` is set.
-        if model.find_unused_parameters:
-            model.reducer.prepare_for_backward(list(_find_tensors(output)))
-        else:
-            model.reducer.prepare_for_backward([])
+        args = list(_find_tensors(output)) if model.find_unused_parameters else []
+        reducer = cast(torch._C._distributed_c10d.Reducer, model.reducer)
+        reducer._rebuild_buckets()  # avoids "INTERNAL ASSERT FAILED" with `find_unused_parameters=False`
+        reducer.prepare_for_backward(args)
     else:
-        model.require_forward_param_sync = False
+        model.require_forward_param_sync = False  # type: ignore[assignment]
 
 
 class UnrepeatedDistributedSampler(DistributedSampler):
-    """A fork of the pytorch DistributedSampler that doesn't repeat data, instead allowing the number of batches
+    """A fork of the PyTorch DistributedSampler that doesn't repeat data, instead allowing the number of batches
     per process to be off-by-one from each other. This makes this sampler usable for predictions (it's
     deterministic and doesn't require shuffling). It is potentially unsafe to use this sampler for training,
     because during training the DistributedDataParallel syncs buffers on each forward pass, so it could freeze if
@@ -92,6 +76,8 @@ class UnrepeatedDistributedSampler(DistributedSampler):
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
+        if not isinstance(self.dataset, Sized):
+            raise TypeError("The given dataset must implement the `__len__` method.")
         self.num_samples = len(range(self.rank, len(self.dataset), self.num_replicas))
         self.total_size = len(self.dataset)
         # If any process has at least one batch, every other process needs to
@@ -99,6 +85,8 @@ class UnrepeatedDistributedSampler(DistributedSampler):
         assert self.num_samples >= 1 or self.total_size == 0
 
     def __iter__(self) -> Iterator[List[int]]:
+        if not isinstance(self.dataset, Sized):
+            raise TypeError("The given dataset must implement the `__len__` method.")
         if self.shuffle:
             # deterministically shuffle based on epoch
             g = torch.Generator()
@@ -159,5 +147,5 @@ class IndexBatchSamplerWrapper:
         return self._sampler.batch_size
 
     @property
-    def sampler(self) -> Sampler:
+    def sampler(self) -> Union[Sampler, Iterable]:
         return self._sampler.sampler
