@@ -11,6 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import torch.nn as nn
+
 import pytorch_lightning as pl
 from pytorch_lightning.overrides.base import _LightningModuleWrapperBase, unwrap_lightning_module
 from pytorch_lightning.utilities import _FAIRSCALE_AVAILABLE
@@ -19,11 +21,11 @@ LightningShardedDataParallel = None
 if _FAIRSCALE_AVAILABLE:
     from fairscale.nn.data_parallel.sharded_ddp import ShardedDataParallel
 
-    class LightningShardedDataParallel(_LightningModuleWrapperBase):
+    class LightningShardedDataParallel(_LightningModuleWrapperBase):  # type: ignore[no-redef]
         # Just do this for later docstrings
         pass
 
-    def unwrap_lightning_module_sharded(wrapped_model) -> "pl.LightningModule":
+    def unwrap_lightning_module_sharded(wrapped_model: nn.Module) -> "pl.LightningModule":
         model = wrapped_model
         if isinstance(model, ShardedDataParallel):
             model = model.module
