@@ -11,51 +11,23 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""XLA Profiler will help you debug and optimize training workload performance for your models using Cloud TPU
-performance tools.
-
-Manual capture via TensorBoard
-
-The following instructions are for capturing trace from a running program
-
-0. This [guide](https://cloud.google.com/tpu/docs/pytorch-xla-performance-profiling-tpu-vm#tpu-vm) will
-help you with the Cloud TPU setup with the required installations
-
-1. Start a TensorBoard Server
-
->> tensorboard --logdir ./tensorboard --port 9001
-
-You could view the TensorBoard output at http://localhost:9001 on your local machine, and then open the
-``PROFILE`` plugin from the top right dropdown or open http://localhost:9001/#profile
-
-2. Once the code you'd like to profile is running, click on ``CAPTURE PROFILE`` button. You could enter
-``localhost:9012`` (default port for XLA Profiler) as the Profile Service URL. Then, you could enter
-the number of milliseconds for the profiling duration, and click ``CAPTURE``
-
-3. Make sure the code is running, while you are trying to capture the traces. Also, it would lead to better
-performance insights if the profiling duration is longer than the step time
-
-4. Once the capture is finished, the page will refresh and you could browse through the insights using the
-``Tools`` dropdown at the top left
-"""
 import logging
 from typing import Dict
 
-from pytorch_lightning.profiler.base import BaseProfiler
-from pytorch_lightning.utilities import _TORCH_GREATER_EQUAL_1_8, _TPU_AVAILABLE
+from pytorch_lightning.profiler.profiler import Profiler
+from pytorch_lightning.utilities import _TPU_AVAILABLE
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 
-if _TPU_AVAILABLE and _TORCH_GREATER_EQUAL_1_8:
+if _TPU_AVAILABLE:
     import torch_xla.debug.profiler as xp
 
 log = logging.getLogger(__name__)
 
 
-class XLAProfiler(BaseProfiler):
+class XLAProfiler(Profiler):
 
-    STEP_FUNCTIONS = {"training_step_and_backward", "validation_step", "test_step", "predict_step"}
+    STEP_FUNCTIONS = {"validation_step", "test_step", "predict_step"}
     RECORD_FUNCTIONS = {
-        "training_step_and_backward",
         "training_step",
         "backward",
         "validation_step",
@@ -64,12 +36,15 @@ class XLAProfiler(BaseProfiler):
     }
 
     def __init__(self, port: int = 9012) -> None:
-        """This Profiler will help you debug and optimize training workload performance for your models using Cloud
-        TPU performance tools."""
+        """XLA Profiler will help you debug and optimize training workload performance for your models using Cloud
+        TPU performance tools.
+
+        Args:
+            port: the port to start the profiler server on. An exception is
+                raised if the provided port is invalid or busy.
+        """
         if not _TPU_AVAILABLE:
             raise MisconfigurationException("`XLAProfiler` is only supported on TPUs")
-        if not _TORCH_GREATER_EQUAL_1_8:
-            raise MisconfigurationException("`XLAProfiler` is only supported with `torch-xla >= 1.8`")
         super().__init__(dirpath=None, filename=None)
         self.port = port
         self._recording_map: Dict = {}
@@ -101,6 +76,3 @@ class XLAProfiler(BaseProfiler):
         else:
             self._step_recoding_map[action_name] += 1
         return self._step_recoding_map[action_name]
-
-    def summary(self) -> str:
-        return ""
