@@ -12,9 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import inspect
-from typing import Callable
+from typing import Callable, Optional
 
 
-def is_param_in_hook_signature(hook_fx: Callable, param: str) -> bool:
+def is_param_in_hook_signature(
+    hook_fx: Callable, param: str, explicit: bool = False, min_args: Optional[int] = None
+) -> bool:
+    """
+    Args:
+        hook_fx: the hook callable
+        param: the name of the parameter to check
+        explicit: whether the parameter has to be explicitly declared
+        min_args: whether the `signature` as at least `min_args` parameters
+    """
     hook_params = list(inspect.signature(hook_fx).parameters)
-    return "args" in hook_params or param in hook_params
+    return (
+        param in hook_params
+        or (not explicit and "args" in hook_params)
+        or (isinstance(min_args, int) and len(hook_params) >= min_args)
+    )
